#include "mpeg.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "avcodec.h"
#include "config.h"

UINT8 *mpg_buf, *pic_dat;
AVFrame *yuv_buf;
static AVCodecContext *codec_context;
static AVPicture pic, temp_pic;
static mpeg_frame cur_frame;
const int width = WIDTH, height = HEIGHT;

void mpeg_init(bool ntsc)
{
  avcodec_init();
  avcodec_register_all();
    
  AVCodec *codec = avcodec_find_encoder(CODEC_ID_MPEG2VIDEO);
  if (!codec) {
    fprintf(stderr, "codec not found\n");
    exit(1);
  }

  codec_context = avcodec_alloc_context();
  avcodec_get_context_defaults(codec_context);

  codec_context->bit_rate = 10000000;
  codec_context->width = width;
  codec_context->height = height;

  if (ntsc) {
    codec_context->frame_rate = 30000;
    codec_context->frame_rate_base = 1001;
  } else {
    codec_context->frame_rate = 25;
    codec_context->frame_rate_base = 1;
  }

  codec_context->gop_size = 0; /* I-Frame */
  codec_context->flags = CODEC_FLAG_LOW_DELAY;
  codec_context->mb_decision=2;
  codec_context->qmin=2;
  codec_context->qmax=4;

  if (avcodec_open(codec_context, codec) < 0) {
    fprintf(stderr, "codec could not be opened\n");
    exit(1);
  }

  /* prepare necessary buffers */
  pic_dat = reinterpret_cast<UINT8 *>(malloc(avpicture_get_size(PIX_FMT_YUV420P, width, height)));
  memset(pic_dat, 0, avpicture_get_size(PIX_FMT_YUV420P, width, height)); 
  avpicture_fill(&pic, pic_dat, PIX_FMT_YUV420P, width, height);

  mpg_buf = reinterpret_cast<UINT8 *>(malloc(5000000)); /* if segfaults are generated in encode_video, raise this size */

  /* prepare an AVPicture out of our buffers */
  yuv_buf = avcodec_alloc_frame();

  /* the lower the better quality, but also larger images => slower to transfer */
  yuv_buf->quality = 0;

  /* copy the data from avpicture to avframe */
  for(int i=0; i<4; ++i){
    yuv_buf->data[i]= pic.data[i];
    yuv_buf->linesize[i]= pic.linesize[i];
  }

  cur_frame.data = mpg_buf; // point to the mpg_buf
}

mpeg_frame mpeg_draw(UINT8* buf)
{
  avpicture_fill(&temp_pic, buf, PIX_FMT_RGBA32, width, height);
  img_convert(&pic, PIX_FMT_YUV420P, &temp_pic, PIX_FMT_RGBA32, width, height);

  //avpicture_fill(&pic, buf, PIX_FMT_YUV420P, width, height);
	
  /* copy the data from avpicture to avframe */
    
  for (int i=0; i<4; ++i){
    yuv_buf->data[i]= pic.data[i];
    yuv_buf->linesize[i]= pic.linesize[i];
  }

  cur_frame.size = avcodec_encode_video(codec_context, mpg_buf, 500000, yuv_buf);
  if (cur_frame.data[cur_frame.size - 1] != 0xb7) {
	/* add sequence end code */
	cur_frame.data[cur_frame.size] = 0x00;
	cur_frame.data[cur_frame.size + 1] = 0x00;
	cur_frame.data[cur_frame.size + 2] = 0x01;
	cur_frame.data[cur_frame.size + 3] = 0xb7;
	cur_frame.size += 4;
  }
  return cur_frame;
}

void mpeg_deinit(){
  avcodec_close(codec_context);
  free(pic_dat);
  free(mpg_buf);
}
