#include "x11osd.h"
#include "config.h"

cX11OsdMenu::cX11OsdMenu(void)
{
  osd = NULL;
  _active = false;
  control = new cX11Control(DISPLAY);
}

cX11OsdMenu::~cX11OsdMenu()
{
  Stop();
  delete osd;
  delete control;
}

void cX11OsdMenu::Stop() {
	if (_active) {
	_active = false;
	Cancel(3);
	}
}

void cX11OsdMenu::Action() {
	_active = true;
	while (_active) {
		Draw();
		usleep(20000);
	}
}


void cX11OsdMenu::Show(void)
{
  osd = cOsdProvider::NewOsd(0, 0);
  if (osd) {
     tArea Area = { 1, 1, 719, 575,  8 };
     osd->SetAreas(&Area, 1);
     osd->Flush();
     Start();
     }
}

void cX11OsdMenu::Draw(void)
{
	Image image;
	cBitmap *bmp = NULL;
	int w, h;
        image.read(MEMFILE);
	image.sample(Geometry(WIDTH-(2*WBORDER),HEIGHT-(2*HBORDER)));
	image.borderColor("black");
	image.border(Geometry(WBORDER,HBORDER));
	image.magick("RGBA");
	image.size(Geometry(WIDTH,HEIGHT));

	w = image.columns();
	h = image.rows();
	bmp = new cBitmap(w, h, image.depth());
	
        const PixelPacket *pix = image.getConstPixels(0, 0, w, h);
	for (int iy = 0; iy < h; ++iy) {
	for (int ix = 0; ix < w; ++ix) {
	tColor col = (~(pix->opacity * 255 / MaxRGB) << 24) | ((pix->red * 255 / MaxRGB) << 16) | ((pix->green * 255 / MaxRGB) << 8) | (pix->blue * 255 / MaxRGB);
	bmp->DrawPixel(ix, iy, col);
	++pix;
	}
	}

	osd->DrawBitmap(1,1,*bmp);
	osd->Flush();
	delete(bmp);
}

eOSState cX11OsdMenu::ProcessKey(eKeys Key){
	eOSState state = cOsdObject::ProcessKey(Key);  
	if (state == osUnknown) {
	switch (Key & ~k_Repeat) {
		case kBack:     return osEnd;
		case kOk:
			control->Mouse(8);
			break;
		case kLeft:
			control->Mouse(1);
			break;
		case kRight:
			control->Mouse(2);
			break;
		case kUp:
			control->Mouse(3);
			break;
		case kDown:
			control->Mouse(4);
			break;
		default:
			return state;
  	}
     state = osContinue;
     }
  return state;
}
