/*
 * x11out.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: x11out.c,v 1.4 2003/09/02 15:59:36 schmitzj Exp $
 */

#include <vdr/plugin.h>
#include <sys/types.h>
#include <signal.h>
#include "x11player.h"
#include "x11osd.h"
#include "config.h"

static const char *VERSION        = "0.0.2-beta";
static const char *DESCRIPTION    = "Shows X11 on screen";
static const char *MAINMENUENTRY  = "X11out";

class cPluginX11out : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  int pid;
public:
  cPluginX11out(void);
  virtual ~cPluginX11out();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginX11out::cPluginX11out(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  pid=0;
}

cPluginX11out::~cPluginX11out()
{
  // Clean up after yourself!
	if (pid>0)
	{
		kill(pid,SIGHUP);
		kill(pid,SIGTERM);
 	}
}

const char *cPluginX11out::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginX11out::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginX11out::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginX11out::Start(void)
{
  // Start any background activities the plugin shall perform.
  
  // Xvfb starten

	char cmd[1024];
  
	sprintf(cmd, "mkdir -p '%s'",
					  		   MEMDIR);
	system(cmd);

	sprintf(cmd, "Xvfb %s -fbdir '%s' -screen 0 %sx%d",
							   DISPLAY,
					  		   MEMDIR,
							   XSIZE,
					  		   XCOLOR
			);
	pid = fork();

	if (pid == -1)
		return false;

	if (!pid)
	{ // child
		register int i, mx = getdtablesize();

		for (i = STDERR_FILENO + 1; i < mx; i++)
			close(i); // close all dup()'ed filedescriptors

		execle("/bin/sh", "sh", "-c", cmd, 0, environ);
		exit(127);
	}
	return true;
}

void cPluginX11out::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginX11out::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.

#ifndef FULLSCREENOSD
	x11player *x11p=new x11player();
	x11control *x11c=new x11control(x11p);

	cControl::Launch(x11c);

	return x11c;
#else
	cX11OsdMenu* osd =  new cX11OsdMenu();
	osd->Start();
	return osd;
#endif
}

cMenuSetupPage *cPluginX11out::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginX11out::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}

#include <stdarg.h>

void x11log(int level, const char *fmt, ...)
{
	auto char    t[BUFSIZ];
	auto va_list ap;
	int minlevel=0;

	va_start(ap, fmt);

	if (level>=minlevel)
	{
		vsnprintf(t + 8, sizeof t - 8, fmt, ap);
		memcpy(t, "X11out: ", 8);
		syslog(LOG_DEBUG, "%s", t);
	}
	va_end(ap);
}

VDRPLUGINCREATOR(cPluginX11out); // Don't touch this!
