/*
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: x11player.c,v 1.5 2003/09/02 15:59:36 schmitzj Exp $
 *
 */
#include "x11player.h"
#include "config.h"

x11player::x11player() : cPlayer(), cThread()
{
	doexit=false;
	mpeg_init(false);
	
}
x11player::~x11player()
{
	doexit=true;
	mpeg_deinit();
}

void x11player::Activate(bool o)
{
	Start();
}

void x11player::xwd2mpg()
{
	Image image;
	Blob blob;
	image.read(MEMFILE);
	image.borderColor("black");
	image.sample(Geometry(WIDTH-(2*WBORDER),HEIGHT-(2*HBORDER)));
	image.border(Geometry(WBORDER,HBORDER));
	image.magick("RGBA");
	image.size(Geometry(WIDTH,HEIGHT));
	image.write( &blob );
	UINT8 *frame =  (UINT8 *)blob.data();
	iframe = mpeg_draw(frame);
#ifdef	NOSTILLPICTURE
	send_pes_packet(iframe.data,iframe.size,0);
#else
	DeviceStillPicture(iframe.data,iframe.size);
#endif
}

void x11player::send_pes_packet(unsigned char *data, int len, int timestamp)
{
#define PES_MAX_SIZE 2048
	int ptslen = timestamp ? 5 : 1;
	static unsigned char pes_header[PES_MAX_SIZE];
	
	pes_header[0] = pes_header[1] = 0;
	pes_header[2] = 1;
	pes_header[3] = 0xe0;

	while (len > 0)
	{
		int payload_size = len;
		if (6 + ptslen + payload_size > PES_MAX_SIZE)
			payload_size =PES_MAX_SIZE - (6 + ptslen);
	
		pes_header[4] = (ptslen + payload_size) >> 8;
		pes_header[5] = (ptslen + payload_size) & 255;

		if (ptslen == 5)
		{
			int x;
			x = (0x02 << 4) | (((timestamp >> 30) & 0x07) << 1) | 1;
			pes_header[8] = x;
			x = ((((timestamp >> 15) &0x7fff) << 1) | 1);
			pes_header[7] = x >> 8;
			pes_header[8] = x & 255;
			x = ((((timestamp) & 0x7fff) < 1) | 1);
			pes_header[9] = x >> 8;
			pes_header[10] = x & 255;
		}
		else
		{
			pes_header[6] = 0x0f;
		}

		memcpy (&pes_header[6 + ptslen], data, payload_size);
		PlayPes(pes_header, 6 + ptslen + payload_size);

		len -= payload_size;
		data += payload_size;
		ptslen = 1;
	}
}
void x11player::Action(void)
{
		while (!doexit)
		{
			xwd2mpg();
			usleep(SLEEP);
		}
}

// ----------------------------------------------------------------------

x11control::x11control(x11player *p) : cControl(p)
{
	control = new cX11Control(DISPLAY);
	player=p;
}
x11control::~x11control()
{
	delete player;
	delete control;
}
void x11control::Hide(void)
{
}
eOSState x11control::ProcessKey(eKeys Key)
{
	eOSState state = cOsdObject::ProcessKey(Key);

	if (state == osUnknown)
	{
		switch (Key & ~k_Repeat)
		{
			case kOk:
				control->Mouse(8);
				break;
            		case kBack:
				return osEnd;
				break;
			case kLeft:
				control->Mouse(1);
				break;
			case kRight:
				control->Mouse(2);
				break;
			case kUp:
				control->Mouse(3);
				break;
			case kDown:
				control->Mouse(4);
				break; 
			default:
				return state;
		}
		state = osContinue;
	}
	return state;
}
